@@ -1,12 +1,15 @@
 
 
 try:
+    import tensorflow as tf
+except (ModuleNotFoundError, ImportError):
+    tf = None
+
+if tf is not None:
     from .private_layers import Conditionalize
     from .private_layers import MCLSTM
     from .private_layers import EALSTM
+    from .attention_layers import SelfAttention
+    from .attention_layers import AttentionLSTM
     from .nbeats_keras import NBeats
-    from .tft_layer import TemporalFusionTransformer
-except (ModuleNotFoundError, ImportError):
-    Conditionalize, NBeats, TemporalFusionTransformer = None, None, None
-    MCLSTM = None
-    EALSTM = None
\ No newline at end of file
+    from .tft_layer import TemporalFusionTransformer
\ No newline at end of file
