@@ -1,7 +1,7 @@
 
 from .interpret import Interpret
 from .visualize import Visualize
-from .utils import ProcessResults
+from .utils import ProcessPredictions
 from .explain import ShapExplainer
 from .explain import LimeExplainer
 from .explain import PermutationImportance
